package traffic;

import java.util.*;
import traffic.object.*;

public class Lane implements Constants {
  public final Road road;
  public final PointObject forward;
  public final PointObject back;
  public final int nth;
  public final int priority;
  private final HashSet m_mvObjOnSet = new HashSet();
  private final Set m_unmodifiableMvObjOnSet = Collections.unmodifiableSet(m_mvObjOnSet);

  public final int dx;
  public final int dy;

  public Lane(Road road, PointObject forward, int nth) {
    this.road = road;
    this.forward = forward;
    this.back = (forward == road.head()) ? road.tail() : road.head();
    this.nth = nth;
    this.priority = (forward == road.head()) ? road.linesToHead() : road.linesToTail();

    int numLines = road.linesToHead() + road.linesToTail();
    double n = - numLines / 2d + 0.5
      + ((forward == road.head()) ? nth : (numLines - 1 - nth));
    if (!DRIVING_DIRECTION_IS_RIGHT)
      n = - n;
    dx = (int) (road.laneDx() * n);
    dy = (int) (road.laneDy() * n);
  }

  public boolean isBlocked() { return nth < road.blockedLines(); }

  public void addMvObjOn(MovingObject mv)    { m_mvObjOnSet.add(mv); }
  public void removeMvObjOn(MovingObject mv) { m_mvObjOnSet.remove(mv); }
  public Set mvObjOnSet() { return m_unmodifiableMvObjOnSet; }

  public Lane adjacentLaneOfSamePriorityRoadBy(PointObject by) {
    Road candidate = null;
    Iterator it = by.links().iterator();
    while (it.hasNext()) {
      Object obj = it.next();
      if (!(obj instanceof Road)  ||  obj == road)
	continue;
      Road rd = (Road) obj;
      int p = (by == forward  ^  by == rd.head())
	? rd.linesToHead()
	: rd.linesToTail();
      if (p == priority)
	if (candidate == null) candidate = rd;
	else                   return null;
    }
    if (candidate == null)
      return null;
    return (by == forward  ^  by == candidate.head())
      ? (Lane) candidate.lanesToHead().get(nth)
      : (Lane) candidate.lanesToTail().get(nth);
  }

  public String toString() { return "(b" + back.id + ")r" + road.id + "(f" + forward.id + ")[" + road.length() + "]@" + nth; }
}
